<?php

namespace Skoiy\Command;

use MongoDB\BSON\UTCDateTime;
use MongoDB\Client;
use Skoiy\Mongo\Collection;
use Skoiy\Mongo\Faker;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

final class SeedMongoDBCommand extends Command
{
    private $authors = [
        [
            'id' => 1,
            'oid' => '60a2c09d7e3e420d55f9aae8',
            'l10n' => ['pt', 'en'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
        [
            'id' => 2,
            'oid' => '60a2c09d7e3e420d55f9aae6',
            'l10n' => ['pt', 'en', 'fr'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
        [
            'id' => 3,
            'oid' => '60a2c09d7e3e420d55f9aae4',
            'l10n' => ['pt', 'en', 'es'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
        [
            'id' => 4,
            'oid' => '60a2c09d7e3e420d55f9aae2',
            'l10n' => ['pt'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
    ];

    private $dossiers = [
        [
            'id' => 1,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6394e126181e4f96fa8b5d72',
            'l10n' => ['pt', 'en'],
        ],
        [
            'id' => 2,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6062f6f87f498e45266ea8b9',
            'l10n' => ['pt', 'en', 'fr']
        ],
        [
            'id' => 3,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6062f6f87f498e45266ea8b7',
            'l10n' => ['es']
        ],
    ];

    private $collections = [
        [
            'id' => 1,
            'oid' => '6394e126181e4f96fa8b5d72',
            'l10n' => ['pt', 'en'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
        [
            'id' => 2,
            'oid' => '6062f6f87f498e45266ea8b9',
            'l10n' => ['pt', 'en', 'fr'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
        [
            'id' => 3,
            'oid' => '6062f6f87f498e45266ea8b7',
            'l10n' => ['es'],
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
        ],
    ];

    private $posts = [
        [
            'id' => 16,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6394e126181e4f96fa8b5d72',
            'l10n' => [
                'pt' => [
                    'id' => 1,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ],
                'en' => [
                    'id' => 2,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ]],
            '_stickers' => [],
            '_authors' => [
                ['id' => 1, 'pid' => 1, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 1],
                ['id' => 2, 'pid' => 2, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 2],
            ],
            '_in' => [
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
            ],
            '_posts' => [
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
            ]
        ],
        [
            'id' => 17,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6062f6f87f498e45266ea8b9',
            'l10n' => [
                'pt' => [
                    'id' => 1,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ],
                'en' => [
                    'id' => 2,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ],
                'fr' => [
                    'id' => 3,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ]],
            '_stickers' => [],
            '_authors' => [
                ['id' => 1, 'pid' => 1, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 1],
                ['id' => 2, 'pid' => 2, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 2],
            ],
            '_in' => [
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
            ],
            '_posts' => [
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
            ]
        ],
        [
            'id' => 18,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6062f6f87f498e45266ea8b7',
            'l10n' => [
                'es' => [
                    'id' => 1,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ]
            ],
            '_stickers' => [],
            '_authors' => [
                ['id' => 1, 'pid' => 1, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 1],
                ['id' => 2, 'pid' => 2, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 2],
            ],
            '_in' => [
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
            ],
            '_posts' => [
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
            ]
        ],
        [
            'id' => 19,
            'createdAt' => '2021/05/04 15:07:15 +0100',
            'updatedAt' => '2021/05/04 15:07:15 +0100',
            'oid' => '6062f6f87f498e45266ea8b7',
            'l10n' => [
                'es' => [
                    'id' => 1,
                    'publishedAt' => '2022/10/16 08:00:06 +0100',
                    'beginAt' => '2022/12/28 00:00:00 +0000'
                ]
            ],
            '_stickers' => [],
            '_authors' => [
                ['id' => 1, 'pid' => 1, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 1],
                ['id' => 2, 'pid' => 2, 'idL10n' => 2, 'isoLanguage' => 'pt', 'position' => 2],
            ],
            '_in' => [
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
                ['idCollection' => 1, 'pidCollection' => 1, 'idDossier' => 1, 'pidDossier' => 1, 'position' => 1],
            ],
            '_posts' => [
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
                ['id' => 1, 'idL10n' => 2, 'isoLanguage' => 2, 'position' => 1],
            ]
        ],
    ];

    private $stickers = [
        [
            'id' => 1,
            'token' => 'xyz',
        ],
        [
            'id' => 2,
            'token' => '83a30d0bce'
        ],
        [
            'id' => 3,
            'token' => '83a30d0bce1'
        ],
    ];

    private $stickersPosts = [
        [
            'id' => 1,
            'token' => ['xyz'],
            'items' => [
                ['id' => 16, 'position' => 1, 'value' => 77, 'fixed' => true],
                ['id' => 17, 'position' => 2, 'value' => 55, 'fixed' => true],
                ['id' => 18, 'position' => 2, 'value' => 66, 'fixed' => true]
            ]
        ],
        [
            'id' => 2,
            'token' => ['83a30d0bce'],
            'items' => [
                ['id' => 17, 'position' => 1, 'value' => 55, 'fixed' => true],
                ['id' => 18, 'position' => 2, 'value' => 66, 'fixed' => true],
            ]
        ],
        [
            'id' => 3,
            'token' => ['83a30d0bce1'],
            'items' => [
                ['id' => 19, 'position' => 1, 'value' => 55, 'fixed' => true],
                ['id' => 18, 'position' => 2, 'value' => 66, 'fixed' => true],
            ]
        ],
    ];

    /**
     * In this method setup command, description, and its parameters
     */
    protected function configure()
    {
        $this->setName('seed-mongodb');
        $this->setDescription('Seed the MongoDB.');
    }

    /**
     * Here all logic happens
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln("Seeding MongoDB...");

        $username = isset($_ENV['MONGO_USERNAME']) ? $_ENV['MONGO_USERNAME'] : "root";
        $password = isset($_ENV['MONGO_PASSWORD']) ? $_ENV['MONGO_PASSWORD'] : "secret";
        $host = isset($_ENV['MONGO_HOSTNAME']) ? $_ENV['MONGO_HOSTNAME'] : "mongodb";
        $port = isset($_ENV['MONGO_PORT']) ? $_ENV['MONGO_PORT'] : "27017";;
        $appName = "posts-api";
        $uri = "mongodb://{$username}:{$password}@{$host}:{$port}/?appName={$appName}";
        $this->client = new Client($uri);

        $faker = new Faker();
        $authors = ['data' => $faker->createFakeAuthors($this->authors), 'collection' => Collection::AUTHORS];
        $collections = ['data' => $faker->createFakeCollections($this->collections), 'collection' => Collection::COLLECTIONS];
        $dossiers = ['data' => $faker->createFakeDossiers($this->dossiers), 'collection' => Collection::DOSSIERS];
        $posts = ['data' => $faker->createFakePosts($this->posts), 'collection' => Collection::POSTS];
        $stickers = ['data' => $faker->createFakeStickers($this->stickers), 'collection' => Collection::STICKERS];
        $stickersPosts = ['data' => $faker->createFakeStickersPosts($this->stickersPosts), 'collection' => Collection::STICKERS_POSTS];
        $items = [$authors, $collections, $dossiers, $posts, $stickers, $stickersPosts];

        foreach ($items as $ite) {
            $collection = $ite['collection'];
            $data = $ite['data'];
            $result = $this->client
                ->selectDatabase('test')
                ->selectCollection($collection)
                ->insertMany($data);
        }

        // return value is important when using CI, to fail the build when the command fails
        // in case of fail: "return self::FAILURE;"
        return self::SUCCESS;
    }
}
