<?php

namespace Skoiy\Mongo;

use MongoDB\BSON\UTCDateTime;

class Faker
{
    public function createFakeCollections($collections)
    {
        $now = new UTCDateTime();
        $fakeCollections = [];
        $i = 0;
        foreach ($collections as $collection) {
            $oid = new \MongoDB\BSON\ObjectId($collection['oid']);
            $fakeCollection = [
                'data' => [
                    'id' => $collection['id'],
                    'publicId' => $collection['id'] + 1000,
                    'slug' => 'slug_' . $collection['id'],
                    'baseUrl' => 'http://baseurl/' . $i,
                    'dossiers' => ['count' => $i + 10],
                    'createdAt' => $collection['createdAt'],
                    'updatedAt' => $collection['updatedAt'],
                    'isoLanguages' => $collection['l10n'],
                    'l10n' => $this->createCollectionFakeL10n($collection['l10n'])
                ],
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
            ];
            $fakeCollections[] = $fakeCollection;
            $i++;
        }

        return $fakeCollections;
    }

    public function createFakeAuthors($authors)
    {
        $now = new UTCDateTime();
        $fakeAuthors = [];
        $i = 0;
        foreach ($authors as $author) {
            $fakeAuthor = [
                'data' => [
                    'id' => $author['id'],
                    'baseUrl' => 'http://baseurl/' . $author['id'],
                    'createdAt' => $author['createdAt'],
                    'updatedAt' => $author['updatedAt'],
                    'isoLanguages' => $author['l10n'],
                    'l10n' => $this->createAuthorFakeL10n($author['l10n'])
                ],
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
            ];
            $fakeAuthors[] = $fakeAuthor;
            $i++;
        }

        return $fakeAuthors;
    }

    public function createAuthorFakeL10n(array $isoLanguages)
    {
        $id = 1;
        $l10ns = [];
        foreach ($isoLanguages as $isoLanguage) {
            $l10ns[] = [
                'id' => $id,
                'isoLanguage' => $isoLanguage,
                'role' => 'role',
                'biography' => 'biography',
                'name' => 'name' . $id,
                'description' => 'description',
                'image' => 'image',
                'imageTitle' => 'imageTitle',
                'imageAlterText' => 'imageAlterText',
                'metaTitle' => 'metaTitle',
                'metaDescription' => 'metaDescription' . $id,
                'metaImage' => 'metaImage' . $id,
                'metaKeywords' => 'metaKeywords' . $id,
                'metaUrl' => 'metaUrl' . $id,
            ];
            $id++;
        }
        return $l10ns;
    }

    public function createCollectionFakeL10n(array $isoLanguages)
    {
        $id = 1;
        $l10ns = [];
        foreach ($isoLanguages as $isoLanguage) {
            $l10ns[] = [
                'id' => $id,
                'isoLanguage' => $isoLanguage,
                'name' => 'name_' . $isoLanguage,
                'description' => 'description',
                'image' => 'image',
                'imageTitle' => 'imageTitle',
                'imageAlterText' => 'imageAlterText',
                'metaTitle' => 'metaTitle',
                'metaDescription' => 'metaDescription' . $id,
                'metaImage' => 'metaImage' . $id,
                'metaKeywords' => 'metaKeywords' . $id,
                'metaUrl' => 'metaUrl' . $id,
                'metaWatermarkImage' => 'metaWatermarkImage' . $id,
                'tags' => [
                    [
                        'id' => 1,
                        'name' => 'name',
                        'slug' => 'name',
                        'count' => 4
                    ]
                ]
            ];
            $id++;
        }
        return $l10ns;
    }

    public function createFakeDosssierTags($idDossier)
    {
        $total = rand(1, 5);
        $tags = [];
        for ($i = 0; $i <= $total; $i++) {
            $tags[] = [
                'id' => $i,
                'name' => 'name_' . $idDossier . '_' . $i,
                'slug' => 'slug_' . $idDossier . '_' . $i,
                'count' => 4
            ];
        }
        return $tags;
    }

    public function createDossierFakeL10n($idDossier, array $isoLanguages)
    {
        $id = 1;
        $l10ns = [];
        foreach ($isoLanguages as $isoLanguage) {
            $l10ns[] = [
                'id' => $id,
                'isoLanguage' => $isoLanguage,
                'name' => 'name_' . $isoLanguage,
                'description' => 'description',
                'image' => 'image',
                'imageTitle' => 'imageTitle',
                'imageAlterText' => 'imageAlterText',
                'metaTitle' => 'metaTitle',
                'metaDescription' => 'metaDescription' . $id,
                'metaImage' => 'metaImage' . $id,
                'metaKeywords' => 'metaKeywords' . $id,
                'metaUrl' => 'metaUrl' . $id,
                'metaWatermarkImage' => 'metaWatermarkImage' . $id,
                'tags' => $this->createFakeDosssierTags($idDossier)
            ];
            $id++;
        }
        return $l10ns;
    }

    public function createFakeDossiers($dossiers)
    {
        $now = new UTCDateTime();
        $fakeCollections = [];
        $i = 0;
        foreach ($dossiers as $dossier) {
            $oid = new \MongoDB\BSON\ObjectId($dossier['oid']);
            $fakeCollection = [
                //'_id' => $oid,
                'data' => [
                    'id' => $dossier['id'],
                    'publicId' => $dossier['id'] + 1000,
                    'slug' => 'slug' . $dossier['id'],
                    'baseUrl' => 'http://baseurl/' . $dossier['id'],
                    'createdAt' => $dossier['createdAt'],
                    'updatedAt' => $dossier['updatedAt'],
                    'isoLanguages' => $dossier['l10n'],
                    'l10n' => $this->createDossierFakeL10n($dossier['id'], $dossier['l10n'])
                ],
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
            ];
            $fakeCollections[] = $fakeCollection;
            $i++;
        }

        return $fakeCollections;
    }

    public function createPostFakeL10n($iso, $l10n, $idPost)
    {
        $id = $l10n['id'];
        return (object)[
            'id' => $id,
            'title' => 'title_' . $id . '_post_' . $idPost,
            'slug' => 'slug_' . $id . '_post_' . $idPost,
            'headline' => 'headline_' . $id . '_post_' . $idPost,
            'isoLanguage' => $iso,
            'draft' => false,
            'description' => 'description' . $id,
            'image' => 'xuz.png',
            'imageCaption' => 'imageCaption',
            'imageWidth' => 450,
            'imageHeight' => 780,
            'imageAlign' => 'center',
            'metadata' => (object)[
                'title' => 'title',
                'description' => 'description',
                'image' => 'image',
                'imageOptions' => [],
                'imageTitle' => '',
                'alterText' => '',
                'keywords' => '',
                'url' => '',
                'playmark' => true,
            ],
            'color' => '#F00',
            'publishedAt' => $l10n['publishedAt'],
            'endAt' => null,
            'beginAt' => $l10n['beginAt'],
            'readTime' => rand(5, 25),
            'summary' => (object)[
                'photos' => 1,
                'albums' => 1,
                'authors' => 1,
                'videos' => 1,
                'labels' => 1,
                'audios' => 1,
                'documents' => 1,
                'bodies' => 1,
                'postsRelated' => 1
            ],
            'bodies' => (object)[
                'text' => 'text',
                'restricted' => false,
                'blockType' => ''
            ],
            'tags' => $this->createFakeDosssierTags($idPost)
        ];
    }

    public function createFakePosts($posts)
    {
        $now = new UTCDateTime();
        $fakeCollections = [];
        $i = 0;
        foreach ($posts as $dossier) {
            $oid = new \MongoDB\BSON\ObjectId($dossier['oid']);

            $l10n = [];
            $isos = [];
            foreach ($dossier['l10n'] as $iso => $value) {
                $isos[] = $iso;
                $l10n[] = $this->createPostFakeL10n($iso, $value, $dossier['id'], $dossier['l10n']);
            }

            $fakeCollection = [
                //'_id' => $oid,
                'data' => [
                    'id' => $dossier['id'],
                    'type' => 0,
                    'publicId' => $i + 1000,
                    'baseUrl' => 'http://baseurl/' . $i,
                    'createdAt' => $dossier['createdAt'],
                    'updatedAt' => $dossier['updatedAt'],
                    'isoLanguages' => $isos,
                    'l10n' => $l10n
                ],
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
                '_stickers' => $dossier['_stickers'],
                '_authors' => $dossier['_authors'],
                '_in' => $dossier['_in'],
                '_posts' => $dossier['_posts']
            ];
            $fakeCollections[] = $fakeCollection;
            $i++;
        }

        return $fakeCollections;
    }

    public function createFakeStickers($stickers)
    {
        $now = new UTCDateTime();
        $fakeCollections = [];
        $i = 0;
        foreach ($stickers as $dossier) {
            $fakeCollection = [
                'data' => [
                    'id' => $dossier['id'],
                    'idGroup' => $dossier['id'] + 1000,
                    'token' => $dossier['token'],
                    'name' => 'name_' . $dossier['id'],
                    'total' => 1,
                    'position' => 2,
                    'image' => '',
                    'imageAlign' => 'center',
                    'maximumItems' => 2,
                    'numberOfRows' => 3,
                    'idFormat' => '0',
                    'link' => 'link',
                    'autoFeed' => true,
                    'autoFeedOrder' => '3',
                    'autoFeedType' => 2,
                    'color' => '#F00',
                    'devices' => ['all', 'mobile', 'desktop']
                ],
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
            ];
            $fakeCollections[] = $fakeCollection;
            $i++;
        }

        return $fakeCollections;
    }

    public function createFakeStickersPosts($stickersPosts)
    {
        $now = new UTCDateTime();
        $fakeCollections = [];
        $i = 0;
        foreach ($stickersPosts as $dossier) {
            $fakeCollection = [
                'token' => $dossier['token'],
                'id' => $dossier['id'],
                'idType' => 1,
                'isActive' => true,
                'isType' => 'posts',
                'items' => $dossier['items'],
                'sort' => 'metrics.impressions',
                'sortField' => 'position',
                'streaming' => true,
                '_rev' => $i,
                '_created_at' => $now,
                '_updated_at' => $now,
            ];
            $fakeCollections[] = $fakeCollection;
            $i++;
        }

        return $fakeCollections;
    }
}
