<?php

namespace Skoiy\Mongo\Repository;

use Illuminate\Support\Collection;
use Skoiy\Mongo\Collection as MongoCollection;
use Skoiy\Mongo\Traits\EagerLoadable;
use Skoiy\Mongo\TypeMaps\GenericDocument;
use Skoiy\Mongo\TypeMaps\In;

class EagerLoadCollections extends EagerLoadable
{
    private $repository;
    private $items;
    private $key = 'collections';

    /**
     * @var Collection
     */
    private $documents;

    /**
     * @var GenericDocument
     */
    private $document;

    public function __construct(Repository $repository, Collection $documents)
    {
        $this->repository = $repository;
        $this->documents = $documents;
    }

    public function process(GenericDocument $document, $key = '_in')
    {
        $this->document = $document;
        $ids = $this->getAllCollectionIds($key);
        $this->loadItems($ids);
        $this->append($key, $this->document);
    }

    public function getAllCollectionIds($value): array
    {
        return $this->documents
            ->pluck($value)
            ->flatten()
            ->map(function ($in) {
                return $in->idCollection;
            })
            ->unique()
            ->values()
            ->toArray();
    }

    public function loadItems(array $in)
    {
        $pipeline = [
            [
                '$match' => [
                    'data.id' => [
                        '$in' => $in
                    ]
                ]
            ]
        ];

        $result = $this
            ->repository
            ->from(MongoCollection::COLLECTIONS)
            ->aggregate($pipeline);

        $this->items = $result->toCollection();
    }

    /**
     * @return Collection
     */
    public function items(): Collection
    {
        return $this->items;
    }

    /**
     * @return string
     */
    public function key(): string
    {
        return $this->key;
    }

    public function append($key, GenericDocument $document)
    {
        $post = $document->data;
        $collectionsToAppend = $this->collectionsToAdd($key, $document);
        $post->add($this->key(), $collectionsToAppend);
    }

    private function collectionsToAdd($key, $postDocument)
    {
        /** @var In $authorsInPostDocument */
        $authorsInPostDocument = $postDocument->{$key};

        $ids = $authorsInPostDocument
            ->map(function ($in) {
                return $in->idCollection;
            })
            ->unique()
            ->values()
            ->toArray();

        return $this->items()->filter(function (\Skoiy\Mongo\TypeMaps\CollectionDocument $collectionDocument) use ($ids) {
            return in_array($collectionDocument->data->id, $ids);
        })->values();
    }
}
