<?php

namespace Skoiy\Mongo\Repository;

use Illuminate\Support\Collection;
use Skoiy\Mongo\Collection as MongoCollection;
use Skoiy\Mongo\Traits\EagerLoadable;
use Skoiy\Mongo\TypeMaps\GenericDocument;
use Skoiy\Mongo\TypeMaps\In;
use Skoiy\Mongo\TypeMaps\PostDocument;

class EagerLoadDossiers extends EagerLoadable
{
    private $repository;
    private $items;
    private $key = 'dossiers';

    /**
     * @var GenericDocument
     */
    private $document;

    /**
     * @var Collection
     */
    private $documents;

    public function __construct(Repository $repository, Collection $documents)
    {
        $this->repository = $repository;
        $this->documents = $documents;
    }

    public function getAllDossierIds($value)
    {
        return $this->documents
            ->pluck($value)
            ->flatten()
            ->map(function ($in) {
                return $in->idDossier;
            })
            ->unique()
            ->values()
            ->toArray();
    }

    public function loadItems(array $in)
    {
        $pipeline = [
            [
                '$match' => [
                    'data.id' => [
                        '$in' => $in
                    ]
                ]
            ]
        ];

        $result = $this
            ->repository
            ->from(MongoCollection::DOSSIERS)
            ->aggregate($pipeline);

        $this->items = $result->toCollection();
    }

    /**
     * @return Collection
     */
    public function items(): Collection
    {
        return $this->items;
    }

    /**
     * @return string
     */
    public function key(): string
    {
        return $this->key;
    }

    public function appendTo()
    {
        $dossiersToAppend = $this->items();
        call_user_func_array([$this->document->data, 'add'], [$this->key(), $dossiersToAppend]);
    }

    public function process(GenericDocument $document, $value = '_id')
    {
        $this->document = $document;
        $ids = $this->getAllDossierIds($value);
        $this->loadItems($ids);
        $this->appendTo();
    }

    public function append(PostDocument $postDocument)
    {
        $post = $postDocument->data;
        $dossiersToAppend = $this->dossiersToAdd($post, $postDocument);
        $post->add($this->key(), $dossiersToAppend);
    }

    private function dossiersToAdd(\Skoiy\Mongo\TypeMaps\Post $post, $postDocument)
    {
        /** @var In $authorsInPostDocument */
        $authorsInPostDocument = $postDocument->_in;

        $ids = $authorsInPostDocument
            ->map(function ($in) {
                return $in->idDossier;
            })
            ->unique()
            ->toArray();

        return $this->items()->filter(function (\Skoiy\Mongo\TypeMaps\DossierDocument $dossierDocument) use ($ids) {
            return in_array($dossierDocument->data->id, $ids);
        })->values();
    }
}
