<?php

namespace Skoiy\Mongo\Repository;

use Illuminate\Support\Collection;
use Skoiy\Mongo\Collection as MongoCollection;
use Skoiy\Mongo\Traits\EagerLoadable;
use Skoiy\Mongo\TypeMaps\GenericDocument;
use Skoiy\Mongo\TypeMaps\In;

class EagerLoadStickers extends EagerLoadable
{
    private $repository;
    private $items;
    private $key = 'stickers';

    /**
     * @var GenericDocument
     */
    private $document;

    /**
     * @var Collection
     */
    private $documents;

    public function __construct(Repository $repository, Collection $documents)
    {
        $this->repository = $repository;
        $this->documents = $documents;
    }

    public function getAllStickersIds($value)
    {
        $stickers = $this->documents
            ->pluck('_stickers');

        if ($stickers->isEmpty()) {
            return $stickers;
        }

        return $stickers
            ->flatten()
            ->filter(function ($sticker) {
                return isset($sticker->id);
            })
            ->map(function ($in) {
                return $in->id;
            })
            ->unique()
            ->values()
            ->toArray();
    }

    public function loadItems(array $in)
    {
        $pipeline = [
            [
                '$match' => [
                    'data.id' => [
                        '$in' => $in
                    ]
                ]
            ]
        ];

        $result = $this
            ->repository
            ->from(MongoCollection::STICKERS)
            ->aggregate($pipeline);

        $this->items = $result->toCollection();
    }

    /**
     * @return Collection
     */
    public function items(): Collection
    {
        return $this->items;
    }

    /**
     * @return string
     */
    public function key(): string
    {
        return $this->key;
    }

    public function process(GenericDocument $document, $value = '_stickers')
    {
        $this->document = $document;
        $ids = $this->getAllStickersIds($value);
        $this->loadItems($ids);
        $this->append($this->document);
    }

    public function append(GenericDocument $postDocument)
    {
        $post = $postDocument->data;
        $stickersToAppend = $this->stickersToAdd($post, $postDocument);
        $post->add($this->key(), $stickersToAppend);
    }

    private function stickersToAdd(\Skoiy\Mongo\TypeMaps\Post $post, $postDocument)
    {
        /** @var In $authorsInPostDocument */
        $authorsInPostDocument = $postDocument->_in;

        $ids = $authorsInPostDocument
            ->map(function ($in) {
                return $in->idCollection;
            })
            ->unique()
            ->toArray();

        return $this->items()->filter(function (\Skoiy\Mongo\TypeMaps\StickerDocument $stickerDocument) use ($ids) {
            return in_array($stickerDocument->data->id, $ids);
        })->values();
    }
}
