<?php

namespace Skoiy\Mongo\Subscribers;

class QueryTimeCollector implements \MongoDB\Driver\Monitoring\CommandSubscriber
{
    private $pendingCommands = [];
    private $queryShapeStats = [];

    /* Creates a query shape out of the filter argument. Right now it only
     * takes the top level fields into account */
    private function createQueryShape(array $filter)
    {
        return json_encode(array_keys($filter));
    }

    public function commandStarted(\MongoDB\Driver\Monitoring\CommandStartedEvent $event): void
    {
        if (array_key_exists('find', (array)$event->getCommand())) {
            $queryShape = $this->createQueryShape((array)$event->getCommand()->filter);
            $this->pendingCommands[$event->getRequestId()] = $queryShape;
        }

        if (array_key_exists('aggregate', (array)$event->getCommand())) {
            $queryShape = $this->createQueryShape((array)$event->getCommand()->pipeline);
            $this->pendingCommands[$event->getRequestId()] = $queryShape;
        }
    }

    public function commandSucceeded(\MongoDB\Driver\Monitoring\CommandSucceededEvent $event): void
    {
        $requestId = $event->getRequestId();
        if (array_key_exists($requestId, $this->pendingCommands)) {
            $this->queryShapeStats[$this->pendingCommands[$requestId]]['count']++;
            $this->queryShapeStats[$this->pendingCommands[$requestId]]['duration'] += $event->getDurationMicros();
            unset($this->pendingCommands[$requestId]);
        }
    }

    public function commandFailed(\MongoDB\Driver\Monitoring\CommandFailedEvent $event): void
    {
        if (array_key_exists($event->getRequestId(), $this->pendingCommands)) {
            unset($this->pendingCommands[$event->getRequestId()]);
        }
    }

    public function __destruct()
    {
        foreach ($this->queryShapeStats as $shape => $stats) {
            echo "Shape: ", $shape, " (", $stats['count'], ")\n  ",
                $stats['duration'] / $stats['count'], "µs\n\n";
        }
    }
}
