<?php

namespace Skoiy\Mongo\TypeMaps;

use Illuminate\Contracts\Support\Arrayable;
use Illuminate\Support\Collection as SupportCollection;
use MongoDB\BSON\Unserializable;

class Dossier implements Unserializable, Arrayable
{
    use \Skoiy\Mongo\Traits\Unserializable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'id', 'publicId', 'slug', 'baseUrl', 'createdAt', 'updatedAt', 'isoLanguages', 'l10n'
    ];

    /** @var SupportCollection */
    protected $l10n;

    public function toArray(): array
    {
        $hasArray = [];
        foreach ($this->fillable as $attr) {
            if ($this->{$attr} instanceof SupportCollection) {
                $hasArray[$attr] = $this->{$attr}->toArray();
            } else if($attr == 'isoLanguages') {
                $hasArray[$attr] = isset($this->isoLanguages) ? $this->isoLanguages : [];
            } else {
                $hasArray[$attr] = $this->{$attr};
            }
        }

        // key collections is appended at runtime
        if (isset($this->collections)) {
            $hasArray['collections'] = $this->collections->toArray();
        }

        return $hasArray;
    }

    public function add($key, $value)
    {
        $this->{$key} = $value;
        return $this;
    }

    public function __get($key)
    {
        return isset($this->{$key}) ? $this->{$key} : null;
    }
}
