<?php

namespace Skoiy\Mongo\TypeMaps;

use MongoDB\BSON\Unserializable;
use Skoiy\Contracts\Localization;

class Post implements Unserializable, Localization
{
    /** @var int */
    protected $id;

    /** @var string */
    protected $baseUrl;

    /** @var array */
    protected $isoLanguages;

    /** @var string */
    protected $createdAt;

    /** @var string */
    protected $updatedAt;

    /** @var int */
    protected $publicId;

    /** @var \Skoiy\Mongo\TypeMaps\L10n */
    protected $l10n;

    /** @var \Skoiy\Mongo\TypeMaps\DossierDocument */
    protected $dossiers;

    /** @var \Skoiy\Mongo\TypeMaps\CollectionDocument */
    protected $collections;

    /** @var  \Skoiy\Mongo\TypeMaps\StickerDocument */
    protected $stickers;

    public function bsonUnserialize(array $data): array
    {
        foreach ($data as $k => $value) {
            $this->$k = $value;
        }
        $this->unserialized = true;
        return $data;
    }

    public function toArray(): array
    {
        $data = [
            'id' => $this->id,
            'baseUrl' => $this->baseUrl,
            'createdAt' => $this->createdAt,
            'updatedAt' => $this->updatedAt,
            'publicId' => $this->publicId,
            'isoLanguages' => $this->isoLanguages,
            'l10n' => $this->l10n->toArray(),
        ];

        // key collections is appended at runtime
        if (isset($this->collections)) {
            $data['collections'] = $this->collections->toArray();
        }

        // key dossiers is appended at runtime
        if (isset($this->dossiers)) {
            $data['dossiers'] = $this->dossiers->toArray();
        }

        // key dossiers is appended at runtime
        if (isset($this->stickers)) {
            $data['stickers'] = $this->stickers->toArray();
        }

        return $data;
    }

    public function isoLanguages()
    {
        return $this->isoLanguages;
    }

    public function add($key, $value)
    {
        $this->{$key} = $value;
        return $this;
    }

    public function __get($key)
    {
        return isset($this->{$key}) ? $this->{$key} : null;
    }
}
